use std::sync::Arc;

use actix_web::{App, HttpRequest, HttpResponse, HttpServer, middleware, Responder, web};
use actix_web::web::Data;
use clap::Parser;

use base::RelayedPort;
use tcp_relay_server::args::Args;
use tcp_relay_server::relay_ws::relay_ws;

pub async fn hello_route() -> &'static str {
    "Hello world!"
}

pub async fn config_route(req: HttpRequest, data: Data<Arc<Args>>) -> impl Responder {
    let token = req.headers().get("Authorization")
        .map(|t| t.to_str().unwrap_or_default())
        .unwrap_or_default()
        .strip_prefix("Bearer ")
        .unwrap_or_default();

    if !data.tokens.iter().any(|t| t.eq(token)) {
        return HttpResponse::Unauthorized().json("Missing / invalid token");
    }

    HttpResponse::Ok().json(
        data.ports.iter()
            .enumerate()
            .map(|(id, port)| RelayedPort { id, port: port + data.increment_ports })
            .collect::<Vec<_>>()
    )
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));

    let args: Args = Args::parse();
    let args = Arc::new(args);

    if args.ports.is_empty() {
        log::error!("No port to forward!");
        std::process::exit(2);
    }

    if args.tokens.is_empty() {
        log::error!("No tokens specified!");
        std::process::exit(3);
    }

    log::info!("Starting relay on http://{}", args.listen_address);

    let args_clone = args.clone();
    HttpServer::new(move || {
        App::new()
            .wrap(middleware::Logger::default())
            .app_data(Data::new(args_clone.clone()))
            .route("/", web::get().to(hello_route))
            .route("/config", web::get().to(config_route))
            .route("/ws", web::get().to(relay_ws))
    })
        .bind(&args.listen_address)?
        .run()
        .await
}